;**********************************************************************
;   This file is a basic code template for object module code         *
;   generation on the PICmicro PIC16F628A. This file contains the     *
;   basic code building blocks to build upon.  As a project minimum   *
;   the 16F628A.lkr file will also be required for this file to       *
;   correctly build. The .lkr files are located in the MPLAB          *
;   directory.                                                        *
;                                                                     *
;   If interrupts are not used all code presented between the         *
;   code section "INT_VECTOR and code section "MAIN" can be removed.  *
;   In addition the variable assignments for 'w_temp' and             *
;   'status_temp' can be removed.                                     *
;                                                                     *
;   If interrupts are used, as in this template file, the 16F628A.lkr *
;   file will need to be modified as follows: Remove the lines        *
;     CODEPAGE   NAME=vectors  START=0x0      END=0x4      PROTECTED  *
;   and                                                               *
;     SECTION    NAME=STARTUP  ROM=vectors                            *
;   and change the start address of the page0 section from 0x5 to 0x0 *
;                                                                     *
;   Refer to the MPASM User's Guide for additional information on     *
;   features of the assembler and linker (Document DS33014).          *
;                                                                     *
;   Refer to the respective PICmicro data sheet for additional        *
;   information on the instruction set.                               *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Filename:	    nokia10.asm                                       *
;    Date:          Samstag, 19. Mai 2007 17:09:44                    *
;    File Version:                                                    *
;                                                                     *
;    Author:        ffje                                              *
;    Company:       fE                                                *
;                                                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Files required:                                                  *
;                    16F628A.lkr                                      *
;                                                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Notes:  Die ZeichenSatzTabelle hat 5 Spalten,                    *
;            mu noch im Rom angeordnet werden  OK                    *
;                                                                     *
;            interner RC-osci mit 4 MHz         OK                    *
;            RA6 soll clock-out                 OK                    *
;            wozu eeprom ab 0x2100 mit DE	0x00, 0x01, 0x02, 0x03 ?  *
;            PowerUpTimer = OFF                                       *
;            1. HWInit                          OK                    *
;            2. clkdata                         OK                    *
;            3. wrdata & wrcomm                 OK                    *
;            4. setcusor                        OK                    *
;            5. clearram                        OK                    *
;            6. wrchar                          OK                    *
;            7. Programm-Teile mssen zwischen den                    *
;               Tabellen-Teilen angeordnet werden                     *
;            8. geht nicht auf 37 KHz,                                *
;                                                                     *
;                                                                     *
;**********************************************************************

	list      p=16F628A           ; list directive to define processor
	#include <p16F628A.inc>       ; processor specific variable definitions

	errorlevel  -302              ; suppress message 302 from list file


	__CONFIG   _CP_OFF & _DATA_CP_OFF & _LVP_OFF & _BOREN_OFF & _MCLRE_ON & _WDT_OFF & _PWRTE_OFF & _INTOSC_OSC_CLKOUT


; '__CONFIG' directive is used to embed configuration word within .asm file.
; The lables following the directive are located in the respective .inc file.
; See data sheet for additional information on configuration word settings.


;***** CONSTANTS DEFINITIONS

p_CLK	EQU		0
p_DIN	EQU		1
p_DC	EQU		2
p_CE	EQU		3
p_RES	EQU		4


;***** VARIABLE DEFINITIONS (examples)

; example of using Shared Uninitialized Data Section
INT_VAR		UDATA_SHR	0x71   
w_temp		RES     1		; variable used for context saving 
status_temp	RES     1		; variable used for context saving

; meine Variablen
DVAR		RES		1
DVAR2		RES		1
BITS8		RES		1
BITCNT		RES		1
CUSORX		RES		1
CUSORY		RES		1
ZEICHEN		RES		1



;**********************************************************************
RESET_VECTOR	CODE	0x000		; processor reset vector
		goto    main              ; go to beginning of program


; erstmal ohne intvector
;INT_VECTOR	CODE	0x004		; interrupt vector location
;		movwf   w_temp            ; save off current W register contents
;		movf	STATUS,w          ; move status register into W register
;		movwf	status_temp       ; save off contents of STATUS register


; erstmal ohne isr
; isr code can go here or be located as a call subroutine elsewhere


;		movf    status_temp,w     ; retrieve copy of STATUS register
;		movwf	STATUS            ; restore pre-isr STATUS register contents
;		swapf   w_temp,f
;		swapf   w_temp,w          ; restore pre-isr W register contents
;		retfie                    ; return from interrupt


main
; remaining code goes here


; erstmal unbedingt machen !!!
	call	HWinit
	call	LCD_init


was_machen
	movlw	0x20
	movwf	ZEICHEN
schreiben
	call	wrchar
	incf	ZEICHEN, F
	btfss	ZEICHEN, 7
	goto	schreiben
	nop
nixtun
	nop
	goto 	nixtun		  ; loop forever, remove this instruction, for test only



wrchar
	movlw	0x20			; testen, ob grer als 0x1F
	subwf	ZEICHEN, W
	btfss	STATUS, C
	return					; weil kleiner als 0x20
	btfsc	ZEICHEN, 7		; testen, ob Bit 7 = 0
	return					; weil grer als 0x7F
	nop
; spalte a holen und wrdata
	movlw	HIGH spalten_a
	movwf	PCLATH
	movlw	0x20			; W = Zeichen - 0x20
	subwf	ZEICHEN, W
	call	spalten_a
	movwf	BITS8
	call	wrdata
; spalte b holen und wrdata
	movlw	HIGH spalten_b_c
	movwf	PCLATH
	movlw	0x20			; W = Zeichen - 0x20
	subwf	ZEICHEN, W
	call	spalten_b_c
	movwf	BITS8
	call	wrdata
; spalte c holen und wrdata
	movlw	HIGH spalten_b_c
	movwf	PCLATH
	movlw	0x40			; W = Zeichen + 0x40
	addwf	ZEICHEN, W
	call	spalten_b_c
	movwf	BITS8
	call	wrdata
; spalte d holen und wrdata
	movlw	HIGH spalten_d_e
	movwf	PCLATH
	movlw	0x20			; W = Zeichen - 0x20
	subwf	ZEICHEN, W
	call	spalten_d_e
	movwf	BITS8
	call	wrdata
; spalte e holen und wrdata
	movlw	HIGH spalten_d_e
	movwf	PCLATH
	movlw	0x40			; W = Zeichen + 0x40
	addwf	ZEICHEN, W
	call	spalten_d_e
	movwf	BITS8
	call	wrdata
; Leerspalte
	clrf	BITS8
	call	wrdata
	return					; from wrchar



clearram
	clrf	CUSORX
	clrf	CUSORY
y_loop
	call	setcusor
x_loop
	clrf	BITS8
	call	wrdata
	incf	CUSORX
	movlw	0x54
	subwf	CUSORX, W
	btfss	STATUS, C
	goto	x_loop
	clrf	CUSORX
	incf	CUSORY
	movlw	0x06
	subwf	CUSORY, W
	btfss	STATUS, C
	goto	y_loop
	return					; from clearram



setcusor
;xtest
	movlw	0x54
	subwf	CUSORX, W
	btfss	STATUS, C
	goto	ytest
	movlw	0x53
	movwf	CUSORX
ytest
	movlw	0x06
	subwf	CUSORY, W
	btfss	STATUS, C
	goto	orcusor
	movlw	0x05
	movwf	CUSORY
orcusor
	movlw	0x80			; setcusor x
	iorwf	CUSORX, W
	movwf	BITS8
	call	wrcomm
	movlw	0x40			; setcusor y
	iorwf	CUSORY, W
	movwf	BITS8
	call	wrcomm	
	return					; from setcusor



wrdata
	bsf		PORTA, p_DC
	goto	writeit
wrcomm
	bcf		PORTA, p_DC
writeit
	bcf		PORTA, p_CE
	call	clkdata
	bsf		PORTA, p_CE
	return					; from wrcomm / wrdata



clkdata
	movlw	0x08
	movwf	BITCNT
clk_down
	bcf		PORTA, p_CLK	; CLK = OFF
	rlf		BITS8, F		; hhstes Bit in carry
	btfss	STATUS, C		; berspringe, wenn carry
	goto	nullmachen
	bsf		PORTA, p_DIN	; wenn carry, dann 1 ausgeben
	goto	clk_rise
nullmachen
	bcf		PORTA, p_DIN	; wenn no-carry, dann 0 ausgeben
clk_rise
	bsf		PORTA, p_CLK	; CLK = ON
	decfsz	BITCNT, F
	goto	clk_down
	return					; from clkdata 



HWinit
	movlw	0x08			;CE = ON, RES, DC, DIN, CLK = OFF
	movwf	PORTA
	movlw	0x07			;Turn comparators off and
	movwf	CMCON			;enable pins for I/O functions
	bcf		STATUS, RP1
	bsf		STATUS, RP0 	;Select Bank1
	clrf	TRISA			;Value used to initialize data direction
							;Set RA<4:0> as outputs
							;TRISA<5> always read as 1.
							;TRISA<7:6> depend on oscillator mode
	bcf		STATUS, RP0 	;Select Bank0
; 150 msec warten
	movlw	0xC3			; set outer delay loop
	movwf	DVAR2
Delay0
	clrf	DVAR			; set inner delay loop (0)
Delay1
	decfsz	DVAR
	goto	Delay1
	
	decfsz	DVAR2
	goto	Delay0

	bsf		PORTA, p_RES	; RES = ON
	return					; from HWinit



LCD_init
	movlw	0x21			; activate chip and H = 1
	movwf	BITS8
	call	wrcomm
	movlw	0xC2			; set LCD-voltage to about 7V (0xC2)
	movwf	BITS8			; mein dunkleres LCD braucht 7,6V (0xCC)
	call	wrcomm
	movlw	0x13			; adjust voltage-bias
	movwf	BITS8
	call	wrcomm
	movlw	0x20			; horizontal adressing H = 0
	movwf	BITS8
	call	wrcomm
	movlw	0x09			; activate all segments
	movwf	BITS8
	call	wrcomm
	call	clearram		; erase all pixel on DDRAM
	movlw	0x08			; blank the display
	movwf	BITS8
	call	wrcomm
	movlw	0x0C			; display normal
	movwf	BITS8
	call	wrcomm
	clrf	CUSORX			; cusor home
	clrf	CUSORY
	call	setcusor
	return					; from LCD_init



; initialize eeprom locations
	; testen, was das soll !!!!! 
EE		CODE	0x2100
		DE	0x00, 0x01, 0x02, 0x03


; ZeichenSatzTabelle
	org			0x059F
spalten_a
	addwf	PCL, F
COLUMN_a
	retlw	0x00
	retlw	0x00
	retlw	0x00
	retlw	0x14
	retlw	0x24
	retlw	0x23
	retlw	0x36
	retlw	0x00
	retlw	0x00
	retlw	0x00
	retlw	0x14
	retlw	0x08
	retlw	0x00
	retlw	0x08
	retlw	0x00
	retlw	0x20
	retlw	0x3E
	retlw	0x00
	retlw	0x42
	retlw	0x21
	retlw	0x18
	retlw	0x27
	retlw	0x3C
	retlw	0x01
	retlw	0x36
	retlw	0x06
	retlw	0x00
	retlw	0x00
	retlw	0x08
	retlw	0x14
	retlw	0x00
	retlw	0x02
	retlw	0x32
	retlw	0x7E
	retlw	0x7F
	retlw	0x3E
	retlw	0x7F
	retlw	0x7F
	retlw	0x7F
	retlw	0x3E
	retlw	0x7F
	retlw	0x00
	retlw	0x20
	retlw	0x7F
	retlw	0x7F
	retlw	0x7F
	retlw	0x7F
	retlw	0x3E
	retlw	0x7F
	retlw	0x3E
	retlw	0x7F
	retlw	0x46
	retlw	0x01
	retlw	0x3F
	retlw	0x1F
	retlw	0x3F
	retlw	0x63
	retlw	0x07
	retlw	0x61
	retlw	0x00
	retlw	0x02
	retlw	0x00
	retlw	0x04
	retlw	0x40
	retlw	0x00
	retlw	0x20
	retlw	0x7F
	retlw	0x38
	retlw	0x38
	retlw	0x38
	retlw	0x08
	retlw	0x0C
	retlw	0x7F
	retlw	0x00
	retlw	0x20
	retlw	0x7F
	retlw	0x00
	retlw	0x7C
	retlw	0x7C
	retlw	0x38
	retlw	0x7C
	retlw	0x08
	retlw	0x7C
	retlw	0x48
	retlw	0x04
	retlw	0x3C
	retlw	0x1C
	retlw	0x3C
	retlw	0x44
	retlw	0x0C
	retlw	0x44
	retlw	0x00
	retlw	0x00
	retlw	0x00
	retlw	0x10
	retlw	0x78


	org			0x063F
spalten_b_c
	addwf	PCL, F
COLUMN_b
	retlw	0x00
	retlw	0x00
	retlw	0x07
	retlw	0x7F
	retlw	0x2A
	retlw	0x13
	retlw	0x49
	retlw	0x05
	retlw	0x1C
	retlw	0x41
	retlw	0x08
	retlw	0x08
	retlw	0x50
	retlw	0x08
	retlw	0x60
	retlw	0x10
	retlw	0x51
	retlw	0x42
	retlw	0x61
	retlw	0x41
	retlw	0x14
	retlw	0x45
	retlw	0x4A
	retlw	0x71
	retlw	0x49
	retlw	0x49
	retlw	0x36
	retlw	0x56
	retlw	0x14
	retlw	0x14
	retlw	0x41
	retlw	0x01
	retlw	0x49
	retlw	0x11
	retlw	0x49
	retlw	0x41
	retlw	0x41
	retlw	0x49
	retlw	0x09
	retlw	0x41
	retlw	0x08
	retlw	0x41
	retlw	0x40
	retlw	0x08
	retlw	0x40
	retlw	0x02
	retlw	0x04
	retlw	0x41
	retlw	0x09
	retlw	0x41
	retlw	0x09
	retlw	0x49
	retlw	0x01
	retlw	0x40
	retlw	0x20
	retlw	0x40
	retlw	0x14
	retlw	0x08
	retlw	0x51
	retlw	0x7F
	retlw	0x04
	retlw	0x41
	retlw	0x02
	retlw	0x40
	retlw	0x01
	retlw	0x54
	retlw	0x48
	retlw	0x44
	retlw	0x44
	retlw	0x54
	retlw	0x7E
	retlw	0x52
	retlw	0x08
	retlw	0x44
	retlw	0x40
	retlw	0x10
	retlw	0x41
	retlw	0x04
	retlw	0x08
	retlw	0x44
	retlw	0x14
	retlw	0x14
	retlw	0x08
	retlw	0x54
	retlw	0x3F
	retlw	0x40
	retlw	0x20
	retlw	0x40
	retlw	0x28
	retlw	0x50
	retlw	0x64
	retlw	0x08
	retlw	0x00
	retlw	0x41
	retlw	0x08
	retlw	0x46

COLUMN_c
	retlw	0x00
	retlw	0x5F
	retlw	0x00
	retlw	0x14
	retlw	0x7F
	retlw	0x08
	retlw	0x55
	retlw	0x03
	retlw	0x22
	retlw	0x22
	retlw	0x3E
	retlw	0x3E
	retlw	0x30
	retlw	0x08
	retlw	0x60
	retlw	0x08
	retlw	0x49
	retlw	0x7F
	retlw	0x51
	retlw	0x45
	retlw	0x12
	retlw	0x45
	retlw	0x49
	retlw	0x09
	retlw	0x49
	retlw	0x49
	retlw	0x36
	retlw	0x36
	retlw	0x22
	retlw	0x14
	retlw	0x22
	retlw	0x51
	retlw	0x79
	retlw	0x11
	retlw	0x49
	retlw	0x41
	retlw	0x41
	retlw	0x49
	retlw	0x09
	retlw	0x49
	retlw	0x08
	retlw	0x7F
	retlw	0x41
	retlw	0x14
	retlw	0x40
	retlw	0x0C
	retlw	0x08
	retlw	0x41
	retlw	0x09
	retlw	0x51
	retlw	0x19
	retlw	0x49
	retlw	0x7F
	retlw	0x40
	retlw	0x40
	retlw	0x38
	retlw	0x08
	retlw	0x70
	retlw	0x49
	retlw	0x41
	retlw	0x08
	retlw	0x41
	retlw	0x01
	retlw	0x40
	retlw	0x02
	retlw	0x54
	retlw	0x44
	retlw	0x44
	retlw	0x44
	retlw	0x54
	retlw	0x09
	retlw	0x52
	retlw	0x04
	retlw	0x7D
	retlw	0x44
	retlw	0x28
	retlw	0x7F
	retlw	0x18
	retlw	0x04
	retlw	0x44
	retlw	0x14
	retlw	0x14
	retlw	0x04
	retlw	0x54
	retlw	0x44
	retlw	0x40
	retlw	0x40
	retlw	0x30
	retlw	0x10
	retlw	0x50
	retlw	0x54
	retlw	0x36
	retlw	0x7F
	retlw	0x36
	retlw	0x08
	retlw	0x41


	org			0x073F
spalten_d_e
	addwf	PCL, F
COLUMN_d
	retlw	0x00
	retlw	0x00
	retlw	0x07
	retlw	0x7F
	retlw	0x2A
	retlw	0x64
	retlw	0x22
	retlw	0x00
	retlw	0x41
	retlw	0x1C
	retlw	0x08
	retlw	0x08
	retlw	0x00
	retlw	0x08
	retlw	0x00
	retlw	0x04
	retlw	0x45
	retlw	0x40
	retlw	0x49
	retlw	0x4B
	retlw	0x7F
	retlw	0x45
	retlw	0x49
	retlw	0x05
	retlw	0x49
	retlw	0x29
	retlw	0x00
	retlw	0x00
	retlw	0x41
	retlw	0x14
	retlw	0x14
	retlw	0x09
	retlw	0x41
	retlw	0x11
	retlw	0x49
	retlw	0x41
	retlw	0x22
	retlw	0x49
	retlw	0x09
	retlw	0x49
	retlw	0x08
	retlw	0x41
	retlw	0x3F
	retlw	0x22
	retlw	0x40
	retlw	0x02
	retlw	0x10
	retlw	0x41
	retlw	0x09
	retlw	0x21
	retlw	0x29
	retlw	0x49
	retlw	0x01
	retlw	0x40
	retlw	0x20
	retlw	0x40
	retlw	0x14
	retlw	0x08
	retlw	0x45
	retlw	0x41
	retlw	0x10
	retlw	0x7F
	retlw	0x02
	retlw	0x40
	retlw	0x04
	retlw	0x54
	retlw	0x44
	retlw	0x44
	retlw	0x48
	retlw	0x54
	retlw	0x01
	retlw	0x52
	retlw	0x04
	retlw	0x40
	retlw	0x3D
	retlw	0x44
	retlw	0x40
	retlw	0x04
	retlw	0x04
	retlw	0x44
	retlw	0x14
	retlw	0x18
	retlw	0x04
	retlw	0x54
	retlw	0x40
	retlw	0x20
	retlw	0x20
	retlw	0x40
	retlw	0x28
	retlw	0x50
	retlw	0x4C
	retlw	0x41
	retlw	0x00
	retlw	0x08
	retlw	0x10
	retlw	0x46

COLUMN_e
	retlw	0x00
	retlw	0x00
	retlw	0x00
	retlw	0x14
	retlw	0x12
	retlw	0x62
	retlw	0x50
	retlw	0x00
	retlw	0x00
	retlw	0x00
	retlw	0x14
	retlw	0x08
	retlw	0x00
	retlw	0x08
	retlw	0x00
	retlw	0x02
	retlw	0x3E
	retlw	0x00
	retlw	0x46
	retlw	0x31
	retlw	0x10
	retlw	0x39
	retlw	0x30
	retlw	0x03
	retlw	0x36
	retlw	0x1E
	retlw	0x00
	retlw	0x00
	retlw	0x00
	retlw	0x14
	retlw	0x08
	retlw	0x06
	retlw	0x3E
	retlw	0x7E
	retlw	0x36
	retlw	0x22
	retlw	0x1C
	retlw	0x41
	retlw	0x01
	retlw	0x7A
	retlw	0x7F
	retlw	0x00
	retlw	0x01
	retlw	0x41
	retlw	0x40
	retlw	0x7F
	retlw	0x7F
	retlw	0x3E
	retlw	0x06
	retlw	0x5E
	retlw	0x46
	retlw	0x31
	retlw	0x01
	retlw	0x3F
	retlw	0x1F
	retlw	0x3F
	retlw	0x63
	retlw	0x07
	retlw	0x43
	retlw	0x00
	retlw	0x20
	retlw	0x00
	retlw	0x04
	retlw	0x40
	retlw	0x00
	retlw	0x78
	retlw	0x38
	retlw	0x20
	retlw	0x7F
	retlw	0x18
	retlw	0x02
	retlw	0x3E
	retlw	0x78
	retlw	0x00
	retlw	0x00
	retlw	0x00
	retlw	0x00
	retlw	0x78
	retlw	0x78
	retlw	0x38
	retlw	0x08
	retlw	0x7C
	retlw	0x08
	retlw	0x20
	retlw	0x20
	retlw	0x7C
	retlw	0x1C
	retlw	0x3C
	retlw	0x44
	retlw	0x3C
	retlw	0x44
	retlw	0x00
	retlw	0x00
	retlw	0x00
	retlw	0x08
	retlw	0x78








		END                       ; directive 'end of program'

